from sensor.Sensor import Sensor
from IconSet import IconSet
from Animator import Animator
from utils.datatypes import *
from utils import i18n

import os
import commands


#
# Iconbar for starters (.desktop files).
#
class StarterBar(Sensor):

    # panel orientation values
    TOP = "top"
    BOTTOM = "bottom"
    LEFT = "left"
    RIGHT = "right"


    def __init__(self):

        global _; _ = i18n.Translator("starterbar-sensor")

        # the set of icons
        self.__icon_set = IconSet()

        # the animator
        self.__animator = Animator()

        # the index of the currently selected icon or -1
        self.__current_index = -1

        # the index of the currently dragged icon or -1 if none is dragged
        self.__dragged_icon = -1

        # the current orientation of the panel
        self.__orientation = self.BOTTOM

        # the jump align for icons
        self.__jump_align = -1

        # the scaling of the panel
        self.__panel_scaling = 0.5

        Sensor.__init__(self)
        self._set_config_type("orientation", TYPE_STRING, self.__orientation)
        self._set_config_type("animation", TYPE_BOOL, 1)
        self._set_config_type("panel-scaling", TYPE_INT, 100)

        self.__icon_set.set_visualizer(self.__on_visualize)

        self._add_timer(0, self.__watch_config)
        self._add_timer(0, self.__icon_set.load_icons)
        self._add_timer(50, self.__animator.animation, self.__on_animate)



    #
    # Visualizes the panel.
    #
    def __on_visualize(self, src):

        self.__set_orientation(self.__get_orientation())
        self.__setup_panel()



    #
    # Visualizes animations.
    #
    def __on_animate(self, zoom, align, jump):

        output = self._new_output()
        for i in xrange(len(zoom)):
            output.set("icon-scale[%(i)d]" % vars(),
                       zoom[i] * self.__panel_scaling)
            if (self.__orientation in [self.TOP, self.BOTTOM]):
                output.set("icon-align-x[%(i)d]" % vars(), 0.5 + align[i])
                output.set("icon-align-y[%(i)d]" % vars(),
                           abs(self.__jump_align + jump[i]))
            elif (self.__orientation in [self.LEFT, self.RIGHT]):
                output.set("icon-align-y[%(i)d]" % vars(), 0.5 + align[i])
                output.set("icon-align-x[%(i)d]" % vars(),
                           abs(self.__jump_align + jump[i]))

        #end for
        self._send_output(output)



    #
    # Watches the configuration for changes.
    # TODO: watching service should be provided in the Sensor API
    #
    def __watch_config(self):

        orientation = self._get_config("orientation")
        scaling = self._get_config("panel-scaling") / 100.0
        if (orientation != self.__get_orientation() or
            scaling != self.__panel_scaling):

            self.__panel_scaling = scaling
            self.__set_orientation(orientation)
            self.__setup_panel()

        self._add_timer(250, self.__watch_config)        




    #
    # Sets up the panel.
    #
    def __setup_panel(self):

        length = self.__icon_set.get_length()
        output = self._new_output()
        for i in xrange(length):
            icon = self.__icon_set.get_icon(i)
            output.set("icon-uri[%(i)d]" % vars(), icon.get_pixmap())
            #output.set("icon-scale[%(i)d]" % vars(), self.__panel_scaling)

            if (self.__orientation in [self.TOP, self.BOTTOM]):
                output.set("icon-align-x[%(i)d]" % vars(), 0.5)
                output.set("icon-align-y[%(i)d]" % vars(),
                           abs(self.__jump_align))
            elif (self.__orientation in [self.LEFT, self.RIGHT]):
                output.set("icon-align-y[%(i)d]" % vars(), 0.5)
                output.set("icon-align-x[%(i)d]" % vars(),
                           abs(self.__jump_align))
        #end for

        output.set("length", length)
        self._send_output(output)
        self.__animator.set_size(length)



    #
    # Sets the orientation of the panel.
    #
    def __set_orientation(self, orientation):

        length = self.__icon_set.get_length()
        distance = int(56 * self.__panel_scaling)
        size = max(32, length * 56)
        output = self._new_output()
        if (orientation == self.TOP):
            layout = "horizontal, %(distance)d" % vars()
            panel = (36, 0, size, 50)
            align_width = 120
            align_height = 80
            self.__jump_align = 0
        elif (orientation == self.BOTTOM):
            layout = "horizontal, %(distance)d" % vars()
            panel = (36, 30, size, 50)
            align_width = 120
            align_height = 80
            self.__jump_align = -1
        elif (orientation == self.LEFT):
            layout = "vertical, %(distance)d" % vars()
            panel = (0, 36, 50, size)
            align_width = 80
            align_height = 120
            self.__jump_align = 0
        elif (orientation == self.RIGHT):
            layout = "vertical, %(distance)d" % vars()
            panel = (32, 36, 50, size)
            align_width = 80
            align_height = 120
            self.__jump_align = -1

        scale = self.__panel_scaling
        output.set("layout", layout)
        output.set("panel-x", panel[0] * scale)
        output.set("panel-y", panel[1] * scale)
        output.set("panel-width", panel[2] * scale)
        output.set("panel-height", panel[3] * scale)
        for i in xrange(self.__icon_set.get_length()):
            output.set("align-width[" + str(i) + "]", align_width * scale)
            output.set("align-height[" + str(i) + "]", align_height * scale)
                
        self._send_output(output)
        self.__orientation = orientation



    #
    # Returns the orientation of the panel.
    #
    def __get_orientation(self):

        return self.__orientation



    #
    # Reacts on user actions.
    #
    def call_action(self, action, path, args = []):

        # enter icon
        if (action == "enter" and self.__dragged_icon == -1):
            index = int(path[-1])
            self.__current_index = index
            if (self._get_config("animation")):
                self.__animator.hilight(index)

        # leave icon panel
        elif (action == "leave"):
            self.__current_index = -1
            self.__animator.unhilight()
            self.__dragged_icon = -1

        # start application
        elif (action == "launch"):
            index = int(path[-1])
            cmd = self.__icon_set.get_icon(index).get_command()
            os.system("cd ~ && " + cmd + " & disown")
            if (self._get_config("animation")):
                self.__animator.jump(index)

        # drag icon
        elif (action == "drag-icon"):
            index = int(path[-1])
            self.__dragged_icon = index
            self.__animator.unhilight()

        # drop dragged icon
        elif (action == "drop-icon"):
            index = int(path[-1])
            if (index != self.__dragged_icon):
                self.__icon_set.swap_icons(index, self.__dragged_icon)
            self.__dragged_icon = -1
            
        # open menu
        elif (action == "menu"):
            removable = (self.__current_index != -1)

            menu = [("New starter...", 1, None, self.__add_starter, []),
                    ("Edit starter...", removable, None, self.__edit_starter,
                     [self.__current_index]),
                    ("Remove starter", removable, None, self.__remove_starter,
                     [self.__current_index])
                    ]
            self._open_menu(menu)



    #
    # Adds a new starter.
    #
    def __add_starter(self, src, *args):

        self.__icon_set.add_icon()



    #
    # Removes the given starter.
    #
    def __remove_starter(self, src, index):

        if (index == -1): return
        self.__icon_set.remove_icon(index)



    #
    # Edits the given starter.
    #
    def __edit_starter(self, src, index):

        if (index == -1): return
        self.__icon_set.edit_icon(index)



    #
    # Returns the configuration GUI.
    #
    def get_configurator(self):

        configurator = self._new_configurator()
        configurator.set_name(_("Panel"))
        configurator.add_title(_("Appearance"))
        configurator.add_option(_("Orientation:"), "orientation",
                                _("The orientation of the panel"),
                                [(_("top"), self.TOP),
                                 (_("bottom"), self.BOTTOM),
                                 (_("left"), self.LEFT),
                                 (_("right"), self.RIGHT)]
                                )
        configurator.add_spin(_("Size:"), "panel-scaling",
                              _("The size of the panel in percents"),
                              10, 500)
        configurator.add_checkbox(_("Show animations"), "animation",
                                  _("Toggles eye candy animations on or off"))
        
        return configurator



def new_sensor(args): return apply(StarterBar, args)
